<?php
/**
 *  @author    Martin Tomasek
 *  @license   https://creativecommons.org/licenses/by-sa/4.0/ CC BY-SA 4.0
 */

include_once('fcommon.php');

class FCategory
{
    private $baseQuery;

    public function __construct($common)
    {
        $this->common = $common;
        $ID_SHOP = Configuration::get('SAMBA_SHOP');
        $ID_LANG = Configuration::get('SAMBA_LANG');

        $custom_order = 'CASE id_lang when ' . $ID_LANG . ' then 100 when 0 then 99 else -1000 end';
        $this->baseQuery = '
            SELECT c.id_parent, c.id_category, c.is_root_category, cl.name, max(' . $custom_order . ') 
            FROM ' . $common->tname('category', 'c') . ' 
            ' . $common->tjoin('category_lang', 'cl') . ' on cl.id_category = c.id_category 
            WHERE cl.id_lang = ' . $ID_LANG . ' and cl.id_shop = ' . $ID_SHOP . ' 
            GROUP BY c.id_category
        ';
    }

    public function genFeed($filePath = null)
    {
        // Determine output method
        $isCron = Configuration::get('SAMBA_CRON') == 1;
        $xmlWriter = new XMLWriter();
        if ($isCron) {
            // Generate to file
            $xmlWriter->openMemory();
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('CATEGORIES');
        } else {
            // Generate live output
            header('Content-Type: text/xml');
            $xmlWriter->openURI('php://output');
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('CATEGORIES');
        }

        $children = array();
        $roots = array();
        $nodes = array();
        $cats = Db::getInstance()->ExecuteS($this->baseQuery);
        foreach ($cats as $c) {
            $parent_id = $c['id_parent'];
            $node = array('id' => $c['id_category'], 'name' => $c['name'], 'url' => $this->common->genUrls('category', $c['id_category']));
            $nodes[$c['id_category']] = $node;
            if ($c['is_root_category']) {
                $roots[] = $node;
            }

            if (!array_key_exists($parent_id, $children)) {
                $children[$parent_id] = array();
            }
            $children[$parent_id][] = $node;
        }
        foreach ($roots as $root) {
            $this->common->subtree($xmlWriter, $root['id'], $cattext = array(), $nodes, $children);
        }

        // End the CATEGORIES element and the document
        $xmlWriter->endElement();
        $xmlWriter->endDocument();
        if ($isCron) {
            file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
        } else {
            $xmlWriter->flush();
        }

        return $nodes;
    }
    public function getCategoryNodes()
    {
        $children = [];
        $roots = [];
        $nodes = [];
        $cats = Db::getInstance()->ExecuteS($this->baseQuery);

        foreach ($cats as $c) {
            $parent_id = $c['id_parent'];
            $node = [
                'id' => $c['id_category'],
                'name' => $c['name'],
                'url' => $this->common->genUrls('category', $c['id_category'])
            ];
            $nodes[$c['id_category']] = $node;
            if ($c['is_root_category']) {
                $roots[] = $node;
            }
            if (!array_key_exists($parent_id, $children)) {
                $children[$parent_id] = [];
            }
            $children[$parent_id][] = $node;
        }

        return $nodes;
    }
}
