<?php
/**
*  @author    Martin Tomasek
*  @copyright DiffSolutions, s.r.o.
*  @license   https://creativecommons.org/licenses/by-sa/4.0/ CC BY-SA 4.0
 */

class Common
{
    public function __construct($config)
    {
        $this->config = $config;
        // $this->dispatcher = Dispatcher::getInstance();
    }

    public function getConfig($name)
    {
        return $this->config[$name];
    }

    public function tname($name, $alias = null)
    {
        if (isset($alias)) {
            return $this->config['DB_PREFIX'].$name.' as '.$alias;
        } else {
            return $this->config['DB_PREFIX'].$name;
        }
    }

    public function tjoin($name, $alias = null)
    {
        $q= ' inner join '.$this->tname($name, null);
        if (isset($alias)) {
            $q.= ' as '.$alias;
        }
        return $q;
    }


    public function tleftjoin($name, $alias = null)
    {
        $q= ' left join '.$this->tname($name, null);
        if (isset($alias)) {
            $q.= ' as '.$alias;
        }
        return $q;
    }


    public function parameter($w, $name, $value)
    {
        if (!empty($value) and (!gettype($value) == 'string' or $value != '0.0000000')) {
            #strings are not converted to floats during processing

            $w->startElement('PARAMETER');
            $w->writeElement('NAME', $name);
            $w->writeElement('VALUE', $value);
            $w->endElement();
        }
    }

    public function subtree($w, $node_id, $cattext = array(), &$nodes = null, &$children = null, $write = true)
    {
        $node = $nodes[$node_id];
        $ct = $cattext; #explicit copy
        $name = $node['name'];
        $ct[] = $name;

        $full_name = implode(' | ', array_slice($ct, 1));
        $node['cattext'] = $full_name;
        if ($write) {
            $w->startElement('ITEM');
            $w->writeElement('TITLE', $name);
            $w->writeElement('URL', $node['url']);
        }
        if (array_key_exists($node_id, $children)) {
            foreach ($children[$node_id] as $child) {
                $this->subtree($w, $child['id'], $ct, $nodes, $children, $write);
            }
        }
        if ($write) {
            $w->endElement();
        }
        $nodes[$node_id] = $node;
    }

    public function dtIso($dt)
    {
        $p = explode(' ', $dt);
        if (count($p)>1) {
            return $p[0].'T'.$dt[1].'Z';
        } else {
            return $dt;
        }
    }

    public function genUrls($type, $id,$id_product_attribute = null)
    {
        switch ($type) {
          case 'product':
            $link = Context::getContext()->link->getProductLink((int)$id);
            break;
          case 'variant':
            $link = Context::getContext()->link->getProductLink((int)$id,null,null,null,null,null,(int)$id_product_attribute,false,false,true);
            break;
          case 'category':
          $link = Context::getContext()->link->getCategoryLink((int)$id);
            break;

          default:
            $link = '';
            break;
        }

        return $link;
    }


    public function imgUrl($img_id, $link_rewrite)
    {
        $image_type = Configuration::get('SAMBA_PRODUCTS_IMAGE_TYPE');
        if (isset($image_type)) {
          (Configuration::get('PS_SSL_ENABLED')) ? $url = 'https://' : $url = 'http://';
          return str_replace('.webp','.jpg',$url.$this->config['link']->getImageLink($link_rewrite, $img_id, $image_type));
        }

        return false;

    }

    public function reducedPrice($product_id, $price, $tax = 0)
    {
        $r=SpecificPrice::getSpecificPrice($product_id, 0, 0, 0, 0, 1);
        $price_before = $price;
        if ($r) {
            if ($r['reduction_type'] == 'percentage') {
                $price = $price * (1-$r['reduction']);
            } else {
                $price = ($price - $r['reduction']);
            }
        }
        return array($price * (1+$tax), $price_before * (1+$tax));
    }

    public function orderStatus($status_id)
    {
        $ORDER_CANCELLED = array(6 => true);
        $ORDER_FINISHED = array(5 => true);

        if (array_key_exists($status_id, $ORDER_FINISHED)) {
            return 'finished';
        }
        if (array_key_exists($status_id, $ORDER_CANCELLED)) {
            return 'canceled';
        }
        return 'created';
    }

    public function groupby($items, $key)
    {
        $is = array();
        $id = null;
        $acc = array();
        foreach ($items as $item) {
            $item_id = $item[$key];
            if ($id != $item_id) {
                if (isset($id)) {
                    $is[$id] = $acc;
                }
                $acc = array();
                $id = $item_id;
            }
            $acc[] = $item;
        }
        $is[$id] = $acc;
        return $is;
    }

    public function maybeFloat($val)
    {
        if (empty($val)) {
            return null;
        }
        $v = (float)($val);
        return $v;
    }

    public function validatePhoneFormat($phone,$CallPrefix){
        // This will delete all unwanted chars from number (expect +)
      $phone_number = filter_var($phone, FILTER_SANITIZE_NUMBER_INT);
      // Check if phone number have prefix +xxx and 11 or 12 digits
      if(preg_match('/^\+[0-9]{1,2}[0-9]{3}[0-9]{7}$/', $phone_number)){
      // Check if phone without prefix have 9 digit
      } else if (preg_match('/^[0-9]{9}$/', $phone_number)) {
        $phone_number = '+' . $CallPrefix . $phone_number;
      } else {
        $phone_number = '';
      }
      return $phone_number;
    }
}
