<?php
/**
 *  @autor    Martin Tomasek
 *  @license   https://creativecommons.org/licenses/by-sa/4.0/ CC BY-SA 4.0
 */

include_once('fcommon.php');

class FCustomer
{
    private $baseQuery;

    public function __construct($common, $date_from)
    {
        if (empty($date_from)) {
            $date_from = '1970-01-01';
        }
        $ID_LANG = Configuration::get('SAMBA_LANG');
        $this->common = $common;
        $this->baseQuery = "
            SELECT 
                c.id_customer, c.firstname, c.lastname, c.email, c.newsletter, 
                c.newsletter_date_add, c.optin, c.birthday, c.date_add, c.deleted, 
                gl.name, a.postcode, a.phone, a.phone_mobile, ggl.name AS group_name,
                cc.call_prefix, cc.iso_code, a.min_id AS min_id, o.date_add AS data_zam
            FROM " . _DB_PREFIX_ . "customer c
            LEFT JOIN (
                SELECT id_customer, MAX(id_address) AS min_id, postcode, phone, phone_mobile, id_country
                FROM " . _DB_PREFIX_ . "address
                GROUP BY id_customer
            ) AS a ON a.id_customer = c.id_customer
            LEFT JOIN " . _DB_PREFIX_ . "country AS cc ON cc.id_country = a.id_country
            LEFT JOIN " . _DB_PREFIX_ . "gender_lang AS gl ON gl.id_gender = c.id_gender
            LEFT JOIN " . _DB_PREFIX_ . "group_lang AS ggl ON ggl.id_group = c.id_default_group AND ggl.id_lang = '" . $ID_LANG . "'
            INNER JOIN (
                SELECT id_customer, MAX(date_add) AS date_add
                FROM " . _DB_PREFIX_ . "orders
                WHERE date_add > '" . $date_from . "'
                GROUP BY id_customer
            ) AS o ON o.id_customer = c.id_customer
            WHERE c.email NOT LIKE '%@allegro%' AND o.date_add IS NOT NULL
        ";
    }

    public function validatePhoneFormat($phone, $CallPrefix)
    {
        $phone_number = filter_var($phone, FILTER_SANITIZE_NUMBER_INT);
        if (preg_match('/^\+[0-9]{1,2}[0-9]{3}[0-9]{7}$/', $phone_number)) {
        } else if (preg_match('/^[0-9]{9}$/', $phone_number)) {
            $phone_number = '+' . $CallPrefix . $phone_number;
        } else {
            $phone_number = '';
        }
        return $phone_number;
    }

    public function genFeedPageNewsletter($xmlWriter)
    {
        $customers = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "emailsubscription");
        foreach ($customers as $c) {
            $email = $c['email'];
            $check = Db::getInstance()->executeS("SELECT COUNT(email) FROM " . _DB_PREFIX_ . "customer WHERE email = '$email'");
            if (!$check[0]['COUNT(email)']) {
                $xmlWriter->startElement('CUSTOMER');
                $xmlWriter->writeElement('CUSTOMER_ID', 'N' . $c['id']);
                $c['email'] = preg_replace('/\s+/', '', $c['email']);
                $xmlWriter->writeElement('EMAIL', $c['email']);
                $xmlWriter->writeElement('NEWSLETTER_FREQUENCY', 'every day');
                $xmlWriter->writeElement('REGISTRATION', $this->common->dtIso($c['newsletter_date_add']));
                $xmlWriter->endElement(); // CUSTOMER
            }
        }
    }

    public function genFeed($filePath = null)
    {
        $pageSize = 10000;
        $lastId = 0;
        $batchCount = 0;

        // Determine output method
        $isCron = Configuration::get('SAMBA_CRON') == 1;
        $xmlWriter = new XMLWriter();
        if ($isCron) {
            // Generate to file
            $xmlWriter->openMemory();
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('CUSTOMERS');
        } else {
            // Generate live output
            header('Content-Type: text/xml');
            $xmlWriter->openURI('php://output');
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('CUSTOMERS');
        }

        // Add customers from emailsubscription table
        $this->genFeedPageNewsletter($xmlWriter);

        while (true) {
            $customers = $this->fetchCustomers($lastId, $pageSize);

            if (empty($customers)) {
                break;
            }

            foreach ($customers as $c) {
                $lastId = $c['id_customer'];
                $phone_mobile = $c['phone_mobile'] ? $c['phone_mobile'] : $c['phone'];
                $phone_mobile_validate = $this->validatePhoneFormat($phone_mobile, $c['call_prefix']);
                $newsletter_date_add = $c['newsletter_date_add'];
                $dateTimeZone = date("Y-m-d\TH:i:sP", strtotime($c['newsletter_date_add']));

                $xmlWriter->startElement('CUSTOMER');
                $xmlWriter->writeElement('FIRST_NAME', $c['firstname']);
                $xmlWriter->writeElement('LAST_NAME', $c['lastname']);
                $xmlWriter->writeElement('CUSTOMER_ID', $c['id_customer']);
                $xmlWriter->writeElement('EMAIL', $c['email']);
                $xmlWriter->writeElement('PHONE', $phone_mobile_validate);
                $xmlWriter->writeElement('ZIP_CODE', $c['postcode']);
                $xmlWriter->writeElement('NEWSLETTER_FREQUENCY', $c['newsletter'] ? 'every day' : 'never');
                if ($c['newsletter_date_add'] != '0000-00-00 00:00:00') {
                    $xmlWriter->writeElement('NLF_TIME', $dateTimeZone);
                }
                $xmlWriter->writeElement('SMS_FREQUENCY', $c['optin'] ? 'every day' : 'never');
                $xmlWriter->writeElement('REGISTRATION', $this->common->dtIso($c['date_add']));
                $xmlWriter->writeElement('PRICE_CATEGORY', $this->common->dtIso($c['group_name']));
                $xmlWriter->startElement('PARAMETERS');
                if(isset($c['name']) AND !empty($c['name'])){
                    $xmlWriter->startElement('PARAMETER');
                    $xmlWriter->writeElement('NAME', 'Gender');
                    $xmlWriter->writeElement('VALUE', $c['name']);
                    $xmlWriter->endElement(); // PARAMETER
                }
                if ($c['birthday'] != '0000-00-00' && $c['birthday'] != '') {
                    $xmlWriter->startElement('PARAMETER');
                    $xmlWriter->writeElement('NAME', 'Birthday');
                    $xmlWriter->writeElement('VALUE', $c['birthday']);
                    $xmlWriter->endElement(); // PARAMETER
                }
                $xmlWriter->startElement('PARAMETER');
                $xmlWriter->writeElement('NAME', "Imie");
                $xmlWriter->writeElement('VALUE', $c['firstname']);
                $xmlWriter->endElement(); // PARAMETER
                $xmlWriter->startElement('PARAMETER');
                $xmlWriter->writeElement('NAME', 'Kraj');
                $xmlWriter->writeElement('VALUE', $c['iso_code']);
                $xmlWriter->endElement(); // PARAMETER
                $xmlWriter->endElement(); // PARAMETERS
                $xmlWriter->endElement(); // CUSTOMER
            }

            // Flush the data to the file every batch
            if ($isCron && ++$batchCount % 10 == 0) {
                file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
            }

            // Break if fewer customers than page size, indicating last batch
            if (count($customers) < $pageSize) {
                break;
            }
        }

        // Final flush to make sure we haven't missed anything
        if ($isCron) {
            file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
        }

        // End the CUSTOMERS element and the document
        $xmlWriter->endElement();
        $xmlWriter->endDocument();
        if ($isCron) {
            file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
        } else {
            $xmlWriter->flush();
        }
    }

    private function fetchCustomers($lastId, $limit)
    {
        $query = $this->baseQuery . " AND c.id_customer > $lastId ORDER BY o.date_add ASC LIMIT $limit";
        $customers = Db::getInstance()->executeS($query);
        return $customers;
    }
}
