<?php
/**
 *  @author    Martin Tomasek
 *  @license   https://creativecommons.org/licenses/by-sa/4.0/ CC BY-SA 4.0
 */

include_once('fcommon.php');
include_once('fcategory.php');

class FProduct
{
    private $baseQuery;

    public function __construct($common)
    {
        $ID_LANG = Configuration::get('SAMBA_LANG');
        $ID_SHOP = Configuration::get('SAMBA_SHOP');
        $this->common = $common;
        $this->baseQuery = "
            SELECT 
                p.*, i.id_image as image, pl.*, 
                cl.link_rewrite as cat_link_rewrite, s.quantity as stock, mfg.name as mfg_desc 
            FROM " . $common->tname('product') . " as p 
            " . $common->tjoin('product_lang', 'pl') . " on pl.id_product = p.id_product 
            " . $common->tjoin('image', 'i') . " on p.id_product = i.id_product 
            " . $common->tjoin('stock_available', 's') . " on s.id_product = p.id_product 
            " . $common->tjoin('product_shop', 'ps') . " on ps.id_product = p.id_product 
            " . $common->tjoin('category_lang', 'cl') . " on cl.id_category = p.id_category_default 
            " . $common->tjoin('manufacturer', 'mfg') . " on mfg.id_manufacturer = p.id_manufacturer 
            inner join (select id_product,min(position) as position from " . $common->tname("image") . " group by id_product) img_pos on img_pos.id_product = i.id_product and img_pos.position = i.position 
            where pl.id_lang = " . $ID_LANG . " and ps.id_shop = " . $ID_SHOP . " and s.id_product_attribute = 0 and cl.id_lang = " . $ID_LANG . " 
            group by p.id_product
        ";
    }

    public function genFeed($filePath = null)
    {
        $pageSize = 10000;
        $batchCount = 0;

        // Determine output method
        $isCron = Configuration::get('SAMBA_CRON') == 1;
        $xmlWriter = new XMLWriter();
        if ($isCron) {
            // Generate to file
            $xmlWriter->openMemory();
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('PRODUCTS');
        } else {
            // Generate live output
            header('Content-Type: text/xml');
            $xmlWriter->openURI('php://output');
            $xmlWriter->startDocument('1.0', 'UTF-8');
            $xmlWriter->startElement('PRODUCTS');
        }

        $categories = new FCategory($this->common);
        $cats = $categories->getCategoryNodes(); // returns indexed categories

        $page = 0;
        while (true) {
            $count = $this->genFeedPage($page, $cats, $xmlWriter);
            if ($count < $pageSize) {
                break;
            }
            $page++;
            // Flush the data to the file every batch
            if ($isCron && ++$batchCount % 10 == 0) {
                file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
            }
        }

        // Final flush to make sure we haven't missed anything
        if ($isCron) {
            file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
        }

        // End the PRODUCTS element and the document
        $xmlWriter->endElement();
        $xmlWriter->endDocument();
        if ($isCron) {
            file_put_contents($filePath, $xmlWriter->flush(true), FILE_APPEND);
        } else {
            $xmlWriter->flush();
        }
    }

    private function genFeedPage($page, $cats, $xmlWriter)
    {
        $pagesize = 10000;
        $products = Db::getInstance()->ExecuteS($this->baseQuery . ' ORDER BY p.id_product LIMIT ' . $pagesize . ' OFFSET ' . $page * $pagesize);
        $productsVariants = Configuration::get('SAMBA_PRODUCTS_VARIANTS');
        $this->genProducts($products, $productsVariants, $cats, $xmlWriter);
        return count($products);
    }

    private function genProducts($products, $productsVariants, $cats, $xmlWriter)
    {
        $db = Db::getInstance();
        if ($productsVariants) {
            foreach ($products as $p) {
                $product_id = $p['id_product'];
                $product = new Product((int)$product_id);
                if (!$product->hasAttributes()) {
                    $this->genProduct($p, $productsVariants, $cats, $db, $xmlWriter);
                }
                if ($this->variantsAsProducts($db, $product_id, $p['link_rewrite'], Product::getCover((int)$product_id)['id_image'], $productsVariants, $p, $cats, $xmlWriter)) {
                }
            }
        } else {
            foreach ($products as $p) {
                $this->genProduct($p, $productsVariants, $cats, $db, $xmlWriter);
            }
        }
    }

    private function genProduct($p, $productsVariants, $cats, $db, $xmlWriter)
    {
        $common = $this->common;
        $ID_LANG = Configuration::get('SAMBA_LANG');
        $ID_SHOP = Configuration::get('SAMBA_SHOP');
        $genGroups = Configuration::get('SAMBA_PRODUCTS_GROUP_PRICE');
        $EMPLOYEE_ID = $this->common->getConfig('EMPLOYEE_ID');
        $sGroups = Group::getGroups($ID_LANG);
        $context = Context::getContext();
        $employee = new Employee($EMPLOYEE_ID, $ID_LANG, $ID_SHOP);
        $context->employee = $employee;

        $product_id = $p['id_product'];
        $xmlWriter->startElement('PRODUCT');
        $xmlWriter->writeElement('PRODUCT_ID', $product_id . '-0');
        $xmlWriter->writeElement('TITLE', $p['name']);
        $xmlWriter->writeElement('DESCRIPTION', 'BRAK');
        $xmlWriter->writeElement('URL', $common->genUrls('product', $product_id));
        $xmlWriter->writeElement('IMAGE', $common->imgUrl(Product::getCover((int)$product_id)['id_image'], $p['link_rewrite']));
        if (array_key_exists('cattext', $cats[$p['id_category_default']])) {
            $cattext = ($cats[$p['id_category_default']])['cattext'];
            if ($cattext)
                $xmlWriter->writeElement('CATEGORYTEXT', $cattext);
        }

        // Adding Product Features
        $aGroups = $this->getGroupPrice($sGroups, $product_id, $context);
        if (isset($aGroups) && $genGroups) {
            $xmlWriter->startElement('PRICES');
            foreach ($aGroups as $aGroup) {
                $xmlWriter->startElement('CATEGORY');
                $xmlWriter->writeElement('NAME', $aGroup['name']);
                $xmlWriter->writeElement('PRICE', $aGroup['price']);
                $xmlWriter->writeElement('PRICE_BEFORE_DISCOUNT', $this->price_with_tax((int)$product_id, null, false));
                $xmlWriter->endElement();
            }
            $xmlWriter->endElement();
        }

        $frontfeatures = Product::getFrontFeaturesStatic(1, $product_id);
        if (isset($frontfeatures)) {
            $xmlWriter->startElement('PARAMETERS');
            foreach ($frontfeatures as $feature) {
                $xmlWriter->startElement('PARAMETER');
                $xmlWriter->writeElement('NAME', $feature['name']);
                $xmlWriter->writeElement('VALUE', $feature['value']);
                $xmlWriter->endElement();
            }
            $xmlWriter->endElement();
        }

        $stock = $p['stock'];
        if (!$p['active'])
            $stock = 0;

        if ($p['visibility'] == 'none')
            $stock = 0;

        if ($p['show_price'] <= 0)
            $stock = 0;

        if ($stock < 0)
            $stock = 0;
        $xmlWriter->writeElement('PRICE', $this->price_with_tax((int)$product_id, null, true));
        $xmlWriter->writeElement('PRICE_BEFORE_DISCOUNT', $this->price_with_tax((int)$product_id, null, false));

        $wsp = $p['wholesale_price'];
        if ($wsp && $wsp > 0) {
            $xmlWriter->writeElement('PRICE_BUY', (int)((float)$wsp * 100) / 100);
        } // buy price rounded to 2 decimal places

        $xmlWriter->writeElement('BRAND', $p['mfg_desc']);
        if ($productsVariants) {
            $xmlWriter->writeElement('STOCK', $stock);
        } else {
            if ($this->variants($db, $product_id, $p['link_rewrite'], Product::getCover((int)$product_id)['id_image'], $xmlWriter)) {
                $xmlWriter->writeElement('STOCK', 0);
            }
        }
        $xmlWriter->endElement(); // PRODUCT
    }

    private function variants($db, $product_id, $link_rewrite, $image, $xmlWriter)
    {
        if (empty($product_id)) {
            return false;
        }
        $common = $this->common;
        $q = 'select pa.*, pac.*, sa.quantity as stock from ' .
            $common->tname('product_attribute', 'pa') . ' ' .
            $common->tjoin('product_attribute_combination', 'pac') . ' on pa.id_product_attribute = pac.id_product_attribute ' .
            $common->tjoin('stock_available', 'sa') . ' on pa.id_product_attribute = sa.id_product_attribute ' .
            'where pa.id_product = ' . $product_id . ' ' .
            ' and sa.id_product = ' . $product_id . ' ' .
            'order by pac.id_product_attribute';
        $vs = $db->ExecuteS($q);
        $default_ixs = array();
        foreach ($vs as $ix => $v) {
            if ($v['default_on'] == '1') {
                $default_ixs[] = $ix;
            }
        }
        if ($default_ixs) {
            $default_ix = $default_ixs[0];
            if ($default_ix > 0) {
                $tmp = $vs[$default_ix];
                $vs[$default_ix] = $vs[0];
                $vs[0] = $tmp;
            }
        }

        if (!$vs) {
            return false;
        }
        foreach ($common->groupby($vs, 'id_product_attribute') as $id => $vg) {
            if (empty($vg)) {
                continue;
            }
            $id_product_attribute = $vg[0]['id_product_attribute'];
            $variant_id = $product_id . '-' . $id_product_attribute;
            $xmlWriter->startElement('VARIANT');
            $xmlWriter->writeElement('PRODUCT_ID', $variant_id);
            $xmlWriter->writeElement(
                'PRICE',
                $this->price_with_tax((int)$product_id, (int)$id_product_attribute, true)
            );
            if ($vg[0]['wholesale_price'] > 0)
                $xmlWriter->writeElement(
                    'PRICE_BUY',
                    $vg[0]['wholesale_price']
                );
            $xmlWriter->writeElement(
                'PRICE_BEFORE_DISCOUNT',
                $this->price_with_tax(
                    (int)$product_id,
                    (int)$id_product_attribute,
                    false
                )
            );
            $xmlWriter->writeElement('IMAGE', $common->imgUrl(Product::getCover((int)$product_id)['id_image'], $link_rewrite));
            $xmlWriter->writeElement('URL', $common->genUrls('variant', $product_id, $id_product_attribute));
            $stock = $vg[0]['stock'];
            if ($stock < 0)
                $stock = 0;
            $xmlWriter->writeElement('STOCK', $stock);

            $xmlWriter->endElement(); // VARIANT
        }

        return true;
    }

    private function variantsAsProducts($db, $product_id, $link_rewrite, $image, $productsVariants, $p, $cats, $xmlWriter)
    {
        if (empty($product_id)) {
            return false;
        }
        $ID_LANG = Configuration::get('SAMBA_LANG');
        $ID_SHOP = Configuration::get('SAMBA_SHOP');
        $DESCRIPTION = Configuration::get('SAMBA_DESCRIPTION');
        $genGroups = Configuration::get('SAMBA_PRODUCTS_GROUP_PRICE');
        $EMPLOYEE_ID = $this->common->getConfig('EMPLOYEE_ID');
        $sGroups = Group::getGroups($ID_LANG);
        $context = Context::getContext();
        $employee = new Employee($EMPLOYEE_ID, $ID_LANG, $ID_SHOP);
        $context->employee = $employee;
        $common = $this->common;
        $q = 'select pa.*, pac.*, sa.quantity as stock from ' . $common->tname('product_attribute', 'pa') . ' ' .
            $common->tjoin('product_attribute_combination', 'pac') .
            ' on pa.id_product_attribute = pac.id_product_attribute ' .
            $common->tjoin('stock_available', 'sa') .
            ' on pa.id_product_attribute = sa.id_product_attribute ' .
            'where pa.id_product = ' . $product_id . ' ' .
            ' and sa.id_product = ' . $product_id . ' ' .
            'order by pac.id_product_attribute';
        $vs = $db->ExecuteS($q);
        $default_ixs = array();
        foreach ($vs as $ix => $v) {
            if ($v['default_on'] == '1') {
                $default_ixs[] = $ix;
            }
        }
        if ($default_ixs) {
            $default_ix = $default_ixs[0];
            if ($default_ix > 0) {
                $tmp = $vs[$default_ix];
                $vs[$default_ix] = $vs[0];
                $vs[0] = $tmp;
            }
        }

        if (!$vs) {
            return false;
        }
        foreach ($common->groupby($vs, 'id_product_attribute') as $id => $vg) {
            if (empty($vg)) {
                continue;
            }
            $id_product_attribute = $vg[0]['id_product_attribute'];
            $variant_id = $product_id . '-' . $id_product_attribute;
            $xmlWriter->startElement('PRODUCT');
            $xmlWriter->writeElement('PRODUCT_ID', $variant_id);
            $xmlWriter->writeElement('TITLE', $p['name']);
            $xmlWriter->writeElement('DESCRIPTION', 'BRAK');
            $aGroups = $this->getGroupPrice($sGroups, $product_id, $context, (int)$id_product_attribute);
            if (isset($aGroups) && $genGroups) {
                $xmlWriter->startElement('PRICES');
                foreach ($aGroups as $aGroup) {
                    $xmlWriter->startElement('CATEGORY');
                    $xmlWriter->writeElement('NAME', $aGroup['name']);
                    $xmlWriter->writeElement('PRICE', $aGroup['price']);
                    $xmlWriter->writeElement('PRICE_BEFORE_DISCOUNT', $this->price_with_tax((int)$product_id, (int)$id_product_attribute, false));
                    $xmlWriter->endElement();
                }
                $xmlWriter->endElement();
            }
            $xmlWriter->writeElement(
                'PRICE',
                $this->price_with_tax((int)$product_id, (int)$id_product_attribute, true)
            );
            if ($vg[0]['wholesale_price'] > 0)
                $xmlWriter->writeElement(
                    'PRICE_BUY',
                    $vg[0]['wholesale_price']
                );
            $xmlWriter->writeElement(
                'PRICE_BEFORE_DISCOUNT',
                $this->price_with_tax(
                    (int)$product_id,
                    (int)$id_product_attribute,
                    false
                )
            );
            $frontfeatures = Product::getFrontFeaturesStatic(1, $product_id);
            if (isset($frontfeatures)) {

                $xmlWriter->startElement('PARAMETERS');
                foreach ($frontfeatures as $feature) {
                    $xmlWriter->startElement('PARAMETER');
                    $xmlWriter->writeElement('NAME', $feature['name']);
                    $xmlWriter->writeElement('VALUE', $feature['value']);
                    $xmlWriter->endElement();
                }
                $xmlWriter->endElement();
            }

            $xmlWriter->writeElement('IMAGE', $common->imgUrl(Product::getCover((int)$product_id)['id_image'], $link_rewrite));
            $xmlWriter->writeElement('URL', $common->genUrls('variant', $product_id, $id_product_attribute));
            if (array_key_exists('cattext', $cats[$p['id_category_default']])) {
                $cattext = ($cats[$p['id_category_default']])['cattext'];
                if ($cattext)
                    $xmlWriter->writeElement('CATEGORYTEXT', $cattext);
            }
            $stock = $vg[0]['stock'];
            if ($stock < 0)
                $stock = 0;

            $xmlWriter->writeElement('STOCK', $stock);

            $xmlWriter->endElement(); // PRODUCT
        }

        return true;
    }

    private function getGroupPrice($groups, $id_product, $context, $id_product_attribute = null)
    {
        $aGroup = array();
        $ID_SHOP = Configuration::get('SAMBA_SHOP');
        $ID_CURRENCY = Currency::getDefaultCurrency()->id;
        $ID_COUNTRY = $context->country->id;
        foreach ($groups as $group) {
            $reduction_value = 0;
            $price = Product::getPriceStatic($id_product, true, $id_product_attribute, 2, null, false, null);
            $today = date('y-m-d');
            $reduction = '';
            $spec_reductions = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "specific_price
                WHERE
                id_shop = '$ID_SHOP' AND
                id_product = $id_product AND
                id_product_attribute = '" . $id_product_attribute . "'
            ");
            array_multisort(
                array_map(
                    static function ($element) {
                        return $element['reduction'];
                    },
                    $spec_reductions
                ),
                SORT_DESC,
                $spec_reductions
            );
            $id_group = $group['id_group'];
            if ($group["reduction"] > 0) {
                if ($reduction < $group['reduction']) {
                    $reduction = ($group['reduction'] / 100);
                    $reduction_value = $price * ($reduction);
                    $aGroup[$id_group]['reduction'] = $reduction;
                    $aGroup[$id_group]['name'] = $group['name'];
                    $aGroup[$id_group]['reduction_value'] = $reduction_value;
                    $aGroup[$id_group]['price'] = round($price - $reduction_value, 2);
                }
            } else {
                $aGroup[$id_group]['reduction'] = '';
                $aGroup[$id_group]['name'] = $group['name'];
                $aGroup[$id_group]['reduction_value'] = '';
                $aGroup[$id_group]['price'] = round($price - $reduction_value, 2);
            }
            foreach ($spec_reductions as $spec_reduction) {
                if ((strtotime($spec_reduction['to']) > time() && strtotime($spec_reduction['from']) < time()) || $spec_reduction['to'] == '0000-00-00 00:00:00') {
                    if ($spec_reduction['id_group'] == $id_group || $spec_reduction['id_group'] == 0) {
                        if ($spec_reduction['price'] > 0) {
                            $aGroup[$id_group]['price'] = round($spec_reduction['price'] * (1 + (Tax::getProductTaxRate($id_product) / 100)), 2);
                        } elseif ($spec_reduction['reduction_type'] == 'percentage') {
                            $reduction_cache = $price * $spec_reduction['reduction'];
                            if ($reduction_cache > $reduction_value) {
                                $reduction = $spec_reduction['reduction'];
                                $reduction_value = $reduction_cache;
                                $aGroup[$id_group]['reduction'] = $reduction;
                                $aGroup[$id_group]['name'] = $group['name'];
                                $aGroup[$id_group]['reduction_value'] = $reduction_value;
                                $aGroup[$id_group]['price'] = round($price - $reduction_value, 2);
                            }
                        } elseif ($spec_reduction['reduction_type'] == 'amount') {
                            if ($spec_reduction['reduction'] > $reduction_value) {
                                $reduction_value = $spec_reduction['reduction'];
                                if ($spec_reduction['reduction_tax'] == 0)
                                    $reduction_value *= (1 + (Tax::getProductTaxRate($id_product) / 100));
                                $aGroup[$id_group]['reduction'] = $reduction;
                                $aGroup[$id_group]['name'] = $group['name'];
                                $aGroup[$id_group]['reduction_value'] = $reduction_value;
                                $aGroup[$id_group]['price'] = round($price - $reduction_value, 2);
                            }
                        }
                    }
                }
            }
        }
        return $aGroup;
    }

    private  function price_with_tax($id_product, $id_product_attribute, $use_reduction)
    {
        $specific_price = 0;
        return Product::getPriceStatic(
            (int) $id_product,
            true,
            (int) $id_product_attribute,
            2,
            null,
            false,
            $use_reduction,
            1,
            false,
            null,
            null,
            null,
            $specific_price,
            true,
            true,
            null,
            false
        );
    }
}
